@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: interpret.c,v 1.19 2003/05/19 20:18:45 nalin Exp $"
+#ident "$Id: interpret.c,v 1.20 2003/05/21 21:47:02 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <assert.h>
@@ -171,7 +171,7 @@ main(int argc, char **argv)
 			}
 		}
 		if (values != NULL) {
-			g_value_array_free(values);
+			_vte_matcher_free_params_array(values);
 		}
 		printf(")\n");
 		i += l;
