@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.41 2003/05/19 22:48:56 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.46 2003/06/03 03:11:03 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <errno.h>
@@ -318,8 +318,10 @@ _vte_iso2022_is_ambiguous(gunichar c)
 	return g_tree_lookup(ambiguous, p) == p;
 }
 
+/* If we only have a codepoint, guess what the ambiguous width should be based
+ * on the default region.  Just hope we don't do this too often. */
 static int
-_vte_iso2022_ambiguous_width(void)
+_vte_iso2022_ambiguous_width_guess(void)
 {
 	const char *lang = NULL;
 	int ret = 1;
@@ -355,6 +357,54 @@ _vte_iso2022_ambiguous_width(void)
 	return ret;
 }
 
+/* If we have the encoding, decide how wide an ambiguously-wide character is
+ * based on the encoding.  This is basically what GNU libc does, and it agrees
+ * with my reading of Unicode UAX 11, so.... */
+static int
+_vte_iso2022_ambiguous_width(struct _vte_iso2022_state *state)
+{
+	const char *wide_codelist[] = {
+		"big5",
+		"euccn",
+		"eucjp",
+		"euckr",
+		"euctw",
+		"gb18030",
+		"gb2312",
+		"gbk",
+		"tcvn",
+	};
+	int i, j;
+	char codeset[16];
+
+	/* Catch weirdo cases. */
+	if ((state->codeset == NULL) || (state->codeset[0] == '\0')) {
+		return 1;
+	}
+
+	/* Sort-of canonify the encoding name. */
+	i = j = 0;
+	for (i = 0; state->codeset[i] != '\0'; i++) {
+		if (g_ascii_isalnum(state->codeset[i])) {
+			codeset[j++] = g_ascii_tolower(state->codeset[i]);
+		}
+		if (j >= sizeof(codeset) - 1) {
+			break;
+		}
+	}
+	codeset[j] = '\0';
+
+	/* Check for the name in the list. */
+	for (i = 0; i < G_N_ELEMENTS(wide_codelist); i++) {
+		if (strcmp(codeset, wide_codelist[i]) == 0) {
+			return 2;
+		}
+	}
+
+	/* Not in the list => not wide. */
+	return 1;
+}
+
 static GTree *
 _vte_iso2022_map_init(const struct _vte_iso2022_map *map, gssize length)
 {
@@ -925,10 +975,14 @@ _vte_iso2022_sequence_length(const unsigned char *nextctl, gsize length)
 				}
 				break;
 #endif
+#if 0
 			case '^':
+				/* ESC ^, the PM.  Search for a string
+				 * terminator. */
+#endif
 			case 'P':
-				/* ESC ^, the PM, or ESC P, the DCS.
-				 * Search for a string terminator. */
+				/* ESC P, the DCS.  Search for a string
+				 * terminator. */
 				for (i = 2; i < length - 1; i++) {
 					if ((nextctl[i] == '\033') &&
 					    (nextctl[i + 1] == '\\')) {
@@ -1155,7 +1209,7 @@ process_8_bit_sequence(struct _vte_iso2022_state *state,
 			width = force_width;
 		} else {
 			if (_vte_iso2022_is_ambiguous(c)) {
-				width = _vte_iso2022_ambiguous_width();
+				width = _vte_iso2022_ambiguous_width(state);
 			}
 		}
 #ifdef VTE_DEBUG
@@ -1178,7 +1232,7 @@ static glong
 process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 	      GArray *gunichars)
 {
-	static int ambiguous_width = 0;
+	int ambiguous_width = 0;
 	glong processed = 0;
 	GTree *map;
 	gint bytes_per_char, force_width, current;
@@ -1192,9 +1246,7 @@ process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 	gpointer p;
 	gboolean single, stop;
 
-	if (ambiguous_width == 0) {
-		ambiguous_width = _vte_iso2022_ambiguous_width();
-	}
+	ambiguous_width = _vte_iso2022_ambiguous_width(state);
 
 	single = (state->override != -1);
 	current = (state->override != -1) ? state->override : state->current;
@@ -1265,6 +1317,7 @@ process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 		for (i = 0; &buf[i] < (char*)outbuf; i += sizeof(gunichar)) {
 			c = *(gunichar*)(buf + i);
 			if (c == '\0') {
+				/* Skip the padding character. */
 				continue;
 			}
 			if (_vte_iso2022_is_ambiguous(c)) {
@@ -1543,7 +1596,7 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 #endif
 							break;
 						default:
-							/* See ECMA-35. */
+							/* Application signalled an "identified coding system" we haven't heard of.  See ECMA-35 for gory details. */
 							g_warning(_("Unrecognized identified coding system."));
 							break;
 						}
@@ -1750,6 +1803,7 @@ _vte_iso2022_process(struct _vte_iso2022_state *state,
 #endif
 		_vte_buffer_clear(input);
 	}
+	g_array_free(blocks, TRUE);
 }
 
 gssize
@@ -1757,7 +1811,7 @@ _vte_iso2022_unichar_width(gunichar c)
 {
 	c = c & ~(VTE_ISO2022_ENCODED_WIDTH_MASK); /* just in case */
 	if (_vte_iso2022_is_ambiguous(c)) {
-		return _vte_iso2022_ambiguous_width();
+		return _vte_iso2022_ambiguous_width_guess();
 	}
 	if (g_unichar_iswide(c)) {
 		return 2;
