@@ -19,6 +19,22 @@
 */
 
 #include "../config.h"
+
+#ifdef HAVE_NCURSES
+#include <ncurses.h>
+#include <term.h>
+#define HAVE_CURSES
+#else
+#ifdef HAVE_CURSES
+#include <curses.h>
+#include <term.h>
+#else
+#ifdef HAVE_TERMCAP
+#include <termcap.h>
+#endif
+#endif
+#endif
+
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
@@ -36,24 +52,6 @@
 #include <sys/select.h>
 #endif
 
-#ifdef HAVE_NCURSES
-#include <ncurses.h>
-#ifdef HAVE_TERM_H
-#include <term.h>
-#endif
-#else
-#ifdef HAVE_CURSES
-#include <curses.h>
-#ifdef HAVE_TERM_H
-#include <term.h>
-#endif
-#else
-#ifdef HAVE_TERMCAP
-#include <termcap.h>
-#endif
-#endif
-#endif
-
 #define BUF_SIZE 512
 #define MAX_COLS 512
 
@@ -141,7 +139,7 @@ static void putcap(unsigned char *s) {
 static int do_cs(int y1, int y2) {
   static char temp[16];
   if (ansi_cs) {
-    sprintf(temp, "\e[%d;%dr", y1, y2);
+    sprintf(temp, "%c[%d;%dr", 27, y1, y2);
     write(1, temp, strlen(temp));
   } else putcap((char *)tgoto(t_cs, y2-1, y1-1));
   return 0;
