@@ -19,7 +19,7 @@
 #ifndef vte_vte_h_included
 #define vte_vte_h_included
 
-#ident "$Id: vte.h,v 1.68 2003/05/29 04:50:48 nalin Exp $"
+#ident "$Id: vte.h,v 1.69 2003/06/16 21:15:10 nalin Exp $"
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -325,9 +325,6 @@ glong vte_terminal_get_column_count(VteTerminal *terminal);
 const char *vte_terminal_get_window_title(VteTerminal *terminal);
 const char *vte_terminal_get_icon_title(VteTerminal *terminal);
 
-/* You never saw this. */
-void _vte_terminal_accessible_ref(VteTerminal *terminal);
-
 G_END_DECLS
 
 #endif
