@@ -19,7 +19,7 @@
 /* VTE accessibility object.  Based heavily on inspection of libzvt's
  * accessibility code. */
 
-#ident "$Id: vteaccess.c,v 1.40 2003/05/30 08:55:28 padraigo Exp $"
+#ident "$Id: vteaccess.c,v 1.42 2003/06/16 21:23:02 nalin Exp $"
 
 #include "../config.h"
 
@@ -29,6 +29,7 @@
 #include "debug.h"
 #include "vte.h"
 #include "vteaccess.h"
+#include "vteint.h"
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -131,7 +132,7 @@ xy_from_offset (VteTerminalAccessiblePrivate *priv,
 	gint i;
 	gint linebreak;
 	gint cur_x, cur_y;
-	gint cur_offset;
+	gint cur_offset = 0;
 
 	cur_x = -1;
 	cur_y = -1;
