@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteapp.c,v 1.60 2003/05/20 05:23:54 nalin Exp $"
+#ident "$Id: vteapp.c,v 1.65 2003/05/30 00:35:35 nalin Exp $"
 
 #include "../config.h"
 
@@ -51,10 +51,24 @@ window_title_changed(GtkWidget *widget, gpointer win)
 
 	g_return_if_fail(VTE_TERMINAL(widget));
 	g_return_if_fail(GTK_IS_WINDOW(win));
-	g_return_if_fail(VTE_TERMINAL (widget)->window_title != NULL);
+	g_return_if_fail(VTE_TERMINAL(widget)->window_title != NULL);
 	window = GTK_WINDOW(win);
 
-	gtk_window_set_title(window, VTE_TERMINAL (widget)->window_title);
+	gtk_window_set_title(window, VTE_TERMINAL(widget)->window_title);
+}
+
+static void
+icon_title_changed(GtkWidget *widget, gpointer win)
+{
+	GtkWindow *window;
+
+	g_return_if_fail(VTE_TERMINAL(widget));
+	g_return_if_fail(GTK_IS_WINDOW(win));
+	g_return_if_fail(VTE_TERMINAL(widget)->icon_title != NULL);
+	window = GTK_WINDOW(win);
+
+	g_message("Icon title changed to \"%s\".\n",
+		  VTE_TERMINAL(widget)->icon_title);
 }
 
 static void
@@ -347,7 +361,12 @@ take_xconsole_ownership(GtkWidget *widget, gpointer data)
 
 	name = g_strdup_printf("MIT_CONSOLE_%s", hostname);
 	atom = gdk_atom_intern(name, FALSE);
+#if GTK_CHECK_VERSION(2,2,0)
+	clipboard = gtk_clipboard_get_for_display(gtk_widget_get_display(widget),
+						  atom);
+#else
 	clipboard = gtk_clipboard_get(atom);
+#endif
 	g_free(name);
 
 	gtk_clipboard_set_with_owner(clipboard,
@@ -366,7 +385,8 @@ main(int argc, char **argv)
 	const char *background = NULL;
 	gboolean transparent = FALSE, audible = TRUE, blink = TRUE,
 		 debug = FALSE, dingus = FALSE, geometry = TRUE, dbuffer = TRUE,
-		 console = FALSE, scroll = FALSE;
+		 console = FALSE, scroll = FALSE, keep = FALSE,
+		 icon_title = FALSE;
 	long lines = 100;
 	const char *message = "Launching interactive shell...\r\n";
 	const char *font = NULL;
@@ -378,7 +398,6 @@ main(int argc, char **argv)
 	int i, j;
 	GList *args = NULL;
 	GdkColor fore, back, tint;
-	GdkCursor *gumby = NULL, *hand = NULL;
 	const char *usage = "Usage: %s "
 			    "[ [-B image] | [-T] ] "
 			    "[-D] "
@@ -390,6 +409,8 @@ main(int argc, char **argv)
 			    "[-f font] "
 			    "[-g] "
 			    "[-h] "
+			    "[-i] "
+			    "[-k] "
 			    "[-n] "
 			    "[-t terminaltype]\n";
 	back.red = back.green = back.blue = 0xffff;
@@ -424,7 +445,7 @@ main(int argc, char **argv)
 	g_assert(i < (g_list_length(args) + 2));
 
 	/* Parse some command-line options. */
-	while ((opt = getopt(argc, argv, "B:CDT2abc:df:ghn:st:w:-")) != -1) {
+	while ((opt = getopt(argc, argv, "B:CDT2abc:df:ghkn:st:w:-")) != -1) {
 		gboolean bail = FALSE;
 		switch (opt) {
 			case 'B':
@@ -460,6 +481,12 @@ main(int argc, char **argv)
 			case 'g':
 				geometry = !geometry;
 				break;
+			case 'i':
+				icon_title = !icon_title;
+				break;
+			case 'k':
+				keep = !keep;
+				break;
 			case 'n':
 				lines = atol(optarg);
 				if (lines == 0) {
@@ -495,6 +522,8 @@ main(int argc, char **argv)
 	/* Create a window to hold the scrolling shell, and hook its
 	 * delete event to the quit function.. */
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_container_set_resize_mode(GTK_CONTAINER(window),
+				      GTK_RESIZE_IMMEDIATE);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 GTK_SIGNAL_FUNC(deleted_and_quit), window);
 
@@ -521,6 +550,10 @@ main(int argc, char **argv)
 	 * window's title. */
 	g_signal_connect(G_OBJECT(widget), "window-title-changed",
 			 G_CALLBACK(window_title_changed), window);
+	if (icon_title) {
+		g_signal_connect(G_OBJECT(widget), "icon-title-changed",
+				 G_CALLBACK(icon_title_changed), window);
+	}
 
 	/* Connect to the "eof" signal to quit when the session ends. */
 	g_signal_connect(G_OBJECT(widget), "eof",
@@ -568,6 +601,7 @@ main(int argc, char **argv)
 
 	/* Set some defaults. */
 	vte_terminal_set_audible_bell(VTE_TERMINAL(widget), audible);
+	vte_terminal_set_visible_bell(VTE_TERMINAL(widget), !audible);
 	vte_terminal_set_cursor_blinks(VTE_TERMINAL(widget), blink);
 	vte_terminal_set_scroll_background(VTE_TERMINAL(widget), scroll);
 	vte_terminal_set_scroll_on_output(VTE_TERMINAL(widget), FALSE);
@@ -597,13 +631,11 @@ main(int argc, char **argv)
 	vte_terminal_match_add(VTE_TERMINAL(widget), "abcdefg");
 	if (dingus) {
 		i = vte_terminal_match_add(VTE_TERMINAL(widget), DINGUS1);
-		gumby = gdk_cursor_new(GDK_GUMBY);
-		vte_terminal_match_set_cursor(VTE_TERMINAL(widget), i, gumby);
-		gdk_cursor_unref(gumby);
-		hand = gdk_cursor_new(GDK_HAND1);
+		vte_terminal_match_set_cursor_type(VTE_TERMINAL(widget),
+						   i, GDK_GUMBY);
 		i = vte_terminal_match_add(VTE_TERMINAL(widget), DINGUS2);
-		vte_terminal_match_set_cursor(VTE_TERMINAL(widget), i, hand);
-		gdk_cursor_unref(hand);
+		vte_terminal_match_set_cursor_type(VTE_TERMINAL(widget),
+						   i, GDK_HAND1);
 	}
 
 	if (console) {
@@ -678,5 +710,11 @@ main(int argc, char **argv)
 	g_assert(widget == NULL);
 	g_assert(window == NULL);
 
+	if (keep) {
+		while (TRUE) {
+			sleep(60);
+		}
+	}
+
 	return 0;
 }
