@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtebg.c,v 1.4 2003/05/19 20:55:11 nalin Exp $"
+#ident "$Id: vtebg.c,v 1.6 2003/06/04 18:47:34 nalin Exp $"
 #include "../config.h"
 #include <string.h>
 #include <gtk/gtk.h>
@@ -91,6 +91,16 @@ vte_bg_native_new(GdkWindow *window)
 	return pvt;
 }
 
+static void
+_vte_bg_display_sync(VteBg *bg)
+{
+#if GTK_CHECK_VERSION(2,2,0)
+	gdk_display_sync(gdk_drawable_get_display(bg->native->window));
+#else
+	XSync(GDK_DISPLAY(), FALSE);
+#endif
+}
+
 static GdkPixmap *
 vte_bg_root_pixmap(VteBg *bg)
 {
@@ -125,6 +135,7 @@ vte_bg_root_pixmap(VteBg *bg)
 			g_free(pixmaps);
 		}
 	}
+	_vte_bg_display_sync(bg);
 	gdk_error_trap_pop();
 	return pixmap;
 }
@@ -166,10 +177,8 @@ vte_bg_root_filter(GdkXEvent *xevent, GdkEvent *event, gpointer data)
 {
 	return GDK_FILTER_CONTINUE;
 }
-
 #endif
 
-
 static void
 vte_bg_class_init(VteBgClass *klass, gpointer data)
 {
@@ -495,7 +504,6 @@ vte_bg_get_pixmap(VteBg *bg,
 	GdkBitmap *mask;
 	GdkPixbuf *pixbuf;
 	char *file;
-	int width, height;
 
 	if (bg == NULL) {
 		bg = vte_bg_get();
@@ -527,17 +535,35 @@ vte_bg_get_pixmap(VteBg *bg,
 	switch (source_type) {
 	case VTE_BG_SOURCE_ROOT:
 		if (GDK_IS_PIXMAP(bg->root_pixmap)) {
-			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+			int width, height;
+			/* Tell GTK+ that this foreign pixmap shares the
+			 * root window's colormap. */
 			rcolormap = gdk_drawable_get_colormap(gdk_get_default_root_window());
 			if (gdk_drawable_get_colormap(bg->root_pixmap) == NULL) {
-				gdk_drawable_set_colormap(bg->root_pixmap, rcolormap);
+				gdk_drawable_set_colormap(bg->root_pixmap,
+							  rcolormap);
+			}
+
+			/* Retrieve the pixmap's size. */
+			gdk_error_trap_push();
+			width = height = -1;
+			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+			_vte_bg_display_sync(bg);
+			gdk_error_trap_pop();
+
+			/* If the pixmap gave us a valid size, retrieve its
+			 * contents. */
+			if ((width > 0) && (height > 0)) {
+				gdk_error_trap_push();
+				pixbuf = gdk_pixbuf_get_from_drawable(NULL,
+								      bg->root_pixmap,
+								      NULL,
+								      0, 0,
+								      0, 0,
+								      width, height);
+				_vte_bg_display_sync(bg);
+				gdk_error_trap_pop();
 			}
-			pixbuf = gdk_pixbuf_get_from_drawable(NULL,
-							      bg->root_pixmap,
-							      NULL,
-							      0, 0,
-							      0, 0,
-							      width, height);
 		}
 		break;
 	case VTE_BG_SOURCE_PIXBUF:
@@ -633,17 +659,34 @@ vte_bg_get_pixbuf(VteBg *bg,
 	case VTE_BG_SOURCE_ROOT:
 		if (GDK_IS_PIXMAP(bg->root_pixmap)) {
 			gint width, height;
-			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+
+			/* If the pixmap doesn't have a colormap, tell GTK+ that
+			 * it shares the root window's colormap. */
 			rcolormap = gdk_drawable_get_colormap(gdk_get_default_root_window());
 			if (gdk_drawable_get_colormap(bg->root_pixmap) == NULL) {
 				gdk_drawable_set_colormap(bg->root_pixmap, rcolormap);
 			}
-			pixbuf = gdk_pixbuf_get_from_drawable(NULL,
-							      bg->root_pixmap,
-							      NULL,
-							      0, 0,
-							      0, 0,
-							      width, height);
+
+			/* Read the pixmap's size. */
+			gdk_error_trap_push();
+			width = height = -1;
+			gdk_drawable_get_size(bg->root_pixmap, &width, &height);
+			_vte_bg_display_sync(bg);
+			gdk_error_trap_pop();
+
+			/* If we got a valid size, read the pixmap's
+			 * contents. */
+			if ((width > 0) && (height > 0)) {
+				gdk_error_trap_push();
+				pixbuf = gdk_pixbuf_get_from_drawable(NULL,
+								      bg->root_pixmap,
+								      NULL,
+								      0, 0,
+								      0, 0,
+								      width, height);
+				_vte_bg_display_sync(bg);
+				gdk_error_trap_pop();
+			}
 		}
 		break;
 	case VTE_BG_SOURCE_PIXBUF:
