@@ -21,7 +21,7 @@
 #ifndef vte_vtedraw_h_included
 #define vte_vtedraw_h_included
 
-#ident "$Id: vtedraw.h,v 1.4 2003/05/01 19:56:13 nalin Exp $"
+#ident "$Id: vtedraw.h,v 1.8 2003/06/16 21:15:10 nalin Exp $"
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -64,15 +64,20 @@ struct _vte_draw_impl {
 				     const char *file,
 				     const GdkColor *color,
 				     double saturation);
+	gboolean requires_repaint;
 	void (*clear)(struct _vte_draw *, gint, gint, gint, gint);
 	void (*set_text_font)(struct _vte_draw *, const PangoFontDescription *);
 	int (*get_text_width)(struct _vte_draw *);
 	int (*get_text_height)(struct _vte_draw *);
 	int (*get_text_ascent)(struct _vte_draw *);
+	int (*get_char_width)(struct _vte_draw *, gunichar c, int columns);
 	gboolean (*get_using_fontconfig)(struct _vte_draw *);
 	void (*draw_text)(struct _vte_draw *,
 			  struct _vte_draw_text_request *, gsize,
 			  GdkColor *, guchar);
+	gboolean (*draw_char)(struct _vte_draw *,
+			      struct _vte_draw_text_request *,
+			      GdkColor *, guchar);
 	void (*draw_rectangle)(struct _vte_draw *,
 			       gint, gint, gint, gint,
 			       GdkColor *, guchar);
@@ -98,7 +103,8 @@ void _vte_draw_free(struct _vte_draw *draw);
    implementations may require that this visual/colormap pair be used when
    creating a window, and may fail otherwise. */
 GdkVisual *_vte_draw_get_visual(struct _vte_draw *draw);
-GdkColormap *_vte_draw_get_colormap(struct _vte_draw *draw);
+GdkColormap *_vte_draw_get_colormap(struct _vte_draw *draw,
+				    gboolean maybe_use_default);
 
 /* Begin and end a drawing operation.  If anything is buffered locally, it is
    flushed to the window system when _end() is called. */
@@ -114,6 +120,7 @@ void _vte_draw_set_background_image(struct _vte_draw *draw,
 				    const char *file,
 				    const GdkColor *color,
 				    double saturation);
+gboolean _vte_draw_requires_repaint(struct _vte_draw *draw);
 void _vte_draw_clear(struct _vte_draw *draw,
 		     gint x, gint y, gint width, gint height);
 
@@ -124,12 +131,16 @@ void _vte_draw_set_text_font(struct _vte_draw *draw,
 int _vte_draw_get_text_width(struct _vte_draw *draw);
 int _vte_draw_get_text_height(struct _vte_draw *draw);
 int _vte_draw_get_text_ascent(struct _vte_draw *draw);
+int _vte_draw_get_char_width(struct _vte_draw *draw, gunichar c, int columns);
 gboolean _vte_draw_get_using_fontconfig(struct _vte_draw *draw);
 
 /* Draw text or rectangles. */
 void _vte_draw_text(struct _vte_draw *draw,
 		    struct _vte_draw_text_request *requests, gsize n_requests,
 		    GdkColor *color, guchar alpha);
+gboolean _vte_draw_char(struct _vte_draw *draw,
+			struct _vte_draw_text_request *request,
+			GdkColor *color, guchar alpha);
 void _vte_draw_fill_rectangle(struct _vte_draw *draw,
 			      gint x, gint y, gint width, gint height,
 			      GdkColor *color, guchar alpha);
