@@ -19,9 +19,9 @@
 #ifndef vte_vteglyph_h_included
 #define vte_vteglyph_h_included
 
-#ident "$Id: vteglyph.h,v 1.3 2003/05/05 23:23:46 nalin Exp $"
+#ident "$Id: vteglyph.h,v 1.6 2003/05/28 21:53:59 nalin Exp $"
 
-#include <glib.h>
+#include <gtk/gtk.h>
 #include "vtefc.h"
 #include "vtergb.h"
 
@@ -36,7 +36,7 @@ enum vte_glyph_flags {
 	vte_glyph_underline	= 1 << 2,
 	vte_glyph_underline2	= 1 << 3,
 	vte_glyph_strikethrough	= 1 << 4,
-	vte_glyph_boxed		= 1 << 5,
+	vte_glyph_boxed		= 1 << 5
 };
 
 #define vte_glyph_double_underline \
@@ -67,11 +67,11 @@ struct _vte_glyph_cache {
 struct _vte_glyph_cache *_vte_glyph_cache_new(void);
 void _vte_glyph_cache_free(struct _vte_glyph_cache *cache);
 const FcPattern *_vte_glyph_cache_get_pattern(struct _vte_glyph_cache *cache);
-void _vte_glyph_cache_set_font_description(FcConfig *config,
+void _vte_glyph_cache_set_font_description(GtkWidget *widget, FcConfig *config,
 					   struct _vte_glyph_cache *cache,
 					   const PangoFontDescription *fontdesc,
-				           _vte_fc_defaults_cb defaults_cb,
-				           gpointer defaults_data);
+					   _vte_fc_defaults_cb defaults_cb,
+					   gpointer defaults_data);
 gboolean _vte_glyph_cache_has_char(struct _vte_glyph_cache *cache, gunichar c);
 const struct _vte_glyph *_vte_glyph_get(struct _vte_glyph_cache *cache,
 					gunichar c);
