@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepango.c,v 1.14 2003/05/28 21:53:59 nalin Exp $"
+#ident "$Id: vtepango.c,v 1.15 2003/06/16 21:15:10 nalin Exp $"
 
 #include "../config.h"
 
@@ -360,6 +360,12 @@ _vte_pango_get_text_ascent(struct _vte_draw *draw)
 	return draw->ascent;
 }
 
+static int
+_vte_pango_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	return _vte_pango_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_pango_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -467,6 +473,7 @@ struct _vte_draw_impl _vte_draw_pango = {
 	_vte_pango_get_text_width,
 	_vte_pango_get_text_height,
 	_vte_pango_get_text_ascent,
+	_vte_pango_get_char_width,
 	_vte_pango_get_using_fontconfig,
 	_vte_pango_draw_text,
 	_vte_pango_draw_char,
